@@ -1,27 +1,23 @@
-# coding=utf8
-#    Copyright (C) 2004-2019 by
-#    Aric Hagberg <hagberg@lanl.gov>
-#    Dan Schult <dschult@colgate.edu>
-#    Pieter Swart <swart@lanl.gov>
-#    All rights reserved.
-#    BSD license.
-#
-# Authors: Aric Hagberg (aric.hagberg@gmail.com)
-#          Pieter Swart (swart@lanl.gov)
-#          Sasha Gutfraind (ag362@cornell.edu)
-#          Vincent Gauthier (vgauthier@luxbulb.org)
 """Katz centrality."""
 from math import sqrt
 
 import networkx as nx
 from networkx.utils import not_implemented_for
 
-__all__ = ['katz_centrality', 'katz_centrality_numpy']
+__all__ = ["katz_centrality", "katz_centrality_numpy"]
 
 
-@not_implemented_for('multigraph')
-def katz_centrality(G, alpha=0.1, beta=1.0, max_iter=1000, tol=1.0e-6,
-                    nstart=None, normalized=True, weight=None):
+@not_implemented_for("multigraph")
+def katz_centrality(
+    G,
+    alpha=0.1,
+    beta=1.0,
+    max_iter=1000,
+    tol=1.0e-6,
+    nstart=None,
+    normalized=True,
+    weight=None,
+):
     r"""Compute the Katz centrality for the nodes of the graph G.
 
     Katz centrality computes the centrality for a node based on the centrality
@@ -79,6 +75,7 @@ def katz_centrality(G, alpha=0.1, beta=1.0, max_iter=1000, tol=1.0e-6,
     weight : None or string, optional (default=None)
       If None, all edge weights are considered equal.
       Otherwise holds the name of the edge attribute used as weight.
+      In this measure the weight is interpreted as the connection strength.
 
     Returns
     -------
@@ -101,9 +98,9 @@ def katz_centrality(G, alpha=0.1, beta=1.0, max_iter=1000, tol=1.0e-6,
     >>> import math
     >>> G = nx.path_graph(4)
     >>> phi = (1 + math.sqrt(5)) / 2.0  # largest eigenvalue of adj matrix
-    >>> centrality = nx.katz_centrality(G, 1/phi - 0.01)
+    >>> centrality = nx.katz_centrality(G, 1 / phi - 0.01)
     >>> for n, c in sorted(centrality.items()):
-    ...    print("%d %0.2f" % (n, c))
+    ...     print(f"{n} {c:.2f}")
     0 0.37
     1 0.60
     2 0.60
@@ -145,7 +142,7 @@ def katz_centrality(G, alpha=0.1, beta=1.0, max_iter=1000, tol=1.0e-6,
     .. [2] Leo Katz:
        A New Status Index Derived from Sociometric Index.
        Psychometrika 18(1):39–43, 1953
-       http://phya.snu.ac.kr/~dkim/PRL87278701.pdf
+       https://link.springer.com/content/pdf/10.1007/BF02289026.pdf
     """
     if len(G) == 0:
         return {}
@@ -154,17 +151,18 @@ def katz_centrality(G, alpha=0.1, beta=1.0, max_iter=1000, tol=1.0e-6,
 
     if nstart is None:
         # choose starting vector with entries of 0
-        x = dict([(n, 0) for n in G])
+        x = {n: 0 for n in G}
     else:
         x = nstart
 
     try:
         b = dict.fromkeys(G, float(beta))
-    except (TypeError, ValueError, AttributeError):
+    except (TypeError, ValueError, AttributeError) as e:
         b = beta
         if set(beta) != set(G):
-            raise nx.NetworkXError('beta dictionary '
-                                   'must have a value for every node')
+            raise nx.NetworkXError(
+                "beta dictionary " "must have a value for every node"
+            ) from e
 
     # make up to max_iter iterations
     for i in range(max_iter):
@@ -183,7 +181,7 @@ def katz_centrality(G, alpha=0.1, beta=1.0, max_iter=1000, tol=1.0e-6,
             if normalized:
                 # normalize vector
                 try:
-                    s = 1.0 / sqrt(sum(v**2 for v in x.values()))
+                    s = 1.0 / sqrt(sum(v ** 2 for v in x.values()))
                 # this should never be zero?
                 except ZeroDivisionError:
                     s = 1.0
@@ -195,9 +193,8 @@ def katz_centrality(G, alpha=0.1, beta=1.0, max_iter=1000, tol=1.0e-6,
     raise nx.PowerIterationFailedConvergence(max_iter)
 
 
-@not_implemented_for('multigraph')
-def katz_centrality_numpy(G, alpha=0.1, beta=1.0, normalized=True,
-                          weight=None):
+@not_implemented_for("multigraph")
+def katz_centrality_numpy(G, alpha=0.1, beta=1.0, normalized=True, weight=None):
     r"""Compute the Katz centrality for the graph G.
 
     Katz centrality computes the centrality for a node based on the centrality
@@ -246,6 +243,7 @@ def katz_centrality_numpy(G, alpha=0.1, beta=1.0, normalized=True,
     weight : None or string, optional
       If None, all edge weights are considered equal.
       Otherwise holds the name of the edge attribute used as weight.
+      In this measure the weight is interpreted as the connection strength.
 
     Returns
     -------
@@ -263,9 +261,9 @@ def katz_centrality_numpy(G, alpha=0.1, beta=1.0, normalized=True,
     >>> import math
     >>> G = nx.path_graph(4)
     >>> phi = (1 + math.sqrt(5)) / 2.0  # largest eigenvalue of adj matrix
-    >>> centrality = nx.katz_centrality_numpy(G, 1/phi)
+    >>> centrality = nx.katz_centrality_numpy(G, 1 / phi)
     >>> for n, c in sorted(centrality.items()):
-    ...    print("%d %0.2f" % (n, c))
+    ...     print(f"{n} {c:.2f}")
     0 0.37
     1 0.60
     2 0.60
@@ -300,32 +298,31 @@ def katz_centrality_numpy(G, alpha=0.1, beta=1.0, normalized=True,
     ----------
     .. [1] Mark E. J. Newman:
        Networks: An Introduction.
-       Oxford University Press, USA, 2010, p. 720.
+       Oxford University Press, USA, 2010, p. 173.
     .. [2] Leo Katz:
        A New Status Index Derived from Sociometric Index.
        Psychometrika 18(1):39–43, 1953
-       http://phya.snu.ac.kr/~dkim/PRL87278701.pdf
+       https://link.springer.com/content/pdf/10.1007/BF02289026.pdf
     """
-    try:
-        import numpy as np
-    except ImportError:
-        raise ImportError('Requires NumPy: http://scipy.org/')
+    import numpy as np
+
     if len(G) == 0:
         return {}
     try:
         nodelist = beta.keys()
         if set(nodelist) != set(G):
-            raise nx.NetworkXError('beta dictionary '
-                                   'must have a value for every node')
+            raise nx.NetworkXError(
+                "beta dictionary " "must have a value for every node"
+            )
         b = np.array(list(beta.values()), dtype=float)
     except AttributeError:
         nodelist = list(G)
         try:
             b = np.ones((len(nodelist), 1)) * float(beta)
-        except (TypeError, ValueError, AttributeError):
-            raise nx.NetworkXError('beta must be a number')
+        except (TypeError, ValueError, AttributeError) as e:
+            raise nx.NetworkXError("beta must be a number") from e
 
-    A = nx.adj_matrix(G, nodelist=nodelist, weight=weight).todense().T
+    A = nx.adjacency_matrix(G, nodelist=nodelist, weight=weight).todense().T
     n = A.shape[0]
     centrality = np.linalg.solve(np.eye(n, n) - (alpha * A), b)
     if normalized:
@@ -334,10 +331,3 @@ def katz_centrality_numpy(G, alpha=0.1, beta=1.0, normalized=True,
         norm = 1.0
     centrality = dict(zip(nodelist, map(float, centrality / norm)))
     return centrality
-
-
-# fixture for pytest
-def setup_module(module):
-    import pytest
-    numpy = pytest.importorskip('numpy')
-    scipy = pytest.importorskip('scipy')
